use std::{
    ffi::{c_char, CString},
    sync::RwLock,
};

mod stingray_sdk;

use crate::stingray_sdk::{GetApiFunction, LoggingApi, PluginApi, PluginApiID};

const PLUGIN_NAME: &str = "dt_p2p";

struct Plugin {
    log: Option<LoggingApi>,
}

impl Plugin {
    pub const fn new() -> Self {
        Self { log: None }
    }
}

static PLUGIN: RwLock<Plugin> = RwLock::new(Plugin::new());

#[no_mangle]
pub extern "C" fn get_name() -> *const c_char {
    let s = CString::new(PLUGIN_NAME).expect("Failed to create CString from plugin name");
    s.as_ptr()
}

#[no_mangle]
pub extern "C" fn setup_game(get_engine_api: GetApiFunction) {
    println!("setup_game");

    PLUGIN.log = Some(LoggingApi::get(get_engine_api));
    let log = PLUGIN.log.as_ref().unwrap();

    log.info(
        PLUGIN_NAME,
        format!("Hello, world! This is {}!", PLUGIN_NAME),
    );
}

#[no_mangle]
pub extern "C" fn shutdown_game() {
    println!("shutdown_game");

    // log.info(PLUGIN_NAME, format!("Goodbye, world!", PLUGIN_NAME));
}

#[no_mangle]
pub extern "C" fn get_plugin_api(id: PluginApiID) -> *mut PluginApi {
    if id == PluginApiID::PLUGIN_API_ID {
        let api = PluginApi {
            get_name: Some(get_name),
            setup_game: Some(setup_game),
            shutdown_game: Some(shutdown_game),
            ..Default::default()
        };

        Box::into_raw(Box::new(api))
    } else {
        std::ptr::null_mut()
    }
}
